using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using BlackJackBusinessObjects;

namespace BlackJackWindows
{
    public partial class BlackJackWindowsForm : Form, IBlackJackUI
    {
        IBlackJackGame game;

        public BlackJackWindowsForm()
        {
            InitializeComponent();
            game = new BlackJackGame(this);
        }

        private void showMessage(string message)
        {
            messageLabel.Text = message;
        }

        #region IPocketJackUI Members

        void IBlackJackUI.DoRedraw()
        {
            Refresh();
        }

        void IBlackJackUI.DoSettingBet()
        {
            betGroupBox.BackColor = Color.Salmon;
            decreaseBetButton.Enabled = true;
            increaseBetButton.Enabled = true;
            hitButton.Enabled = false;
            stayButton.Enabled = false;
        }

        void IBlackJackUI.DoStartPlayerHand()
        {
            betGroupBox.BackColor = Color.Gray;
            decreaseBetButton.Enabled = false;
            increaseBetButton.Enabled = false;
            hitButton.Enabled = true;
            stayButton.Enabled = true;
            showMessage("");
        }

        void IBlackJackUI.DoStartDealerHand()
        {
            betGroupBox.BackColor = Color.Gray;
            decreaseBetButton.Enabled = false;
            increaseBetButton.Enabled = false;
            hitButton.Enabled = false;
            stayButton.Enabled = false;
            showMessage("");
        }

        void IBlackJackUI.ShowBets()
        {
            potLabel.Text = "Pot : " + game.Pot.ToString();
            betLabel.Text = "Bet : " + game.Bet.ToString();
        }

        void IBlackJackUI.ShowPlayerBust()
        {
            showMessage("Player Bust: " + game.player.ScoreHand().ToString());
        }

        void IBlackJackUI.ShowDealerBust()
        {
            showMessage("Dealer Bust: " + 
                game.dealer.ScoreHand().ToString());
        }

        void IBlackJackUI.ShowPlayerWins()
        {
            showMessage("Player wins: " + 
                game.player.ScoreHand().ToString() + 
                " vs " + game.dealer.ScoreHand().ToString());
        }

        void IBlackJackUI.ShowPlayerLoses()
        {
            showMessage("Player loses: " + 
                game.player.ScoreHand().ToString() + " vs " + 
                game.dealer.ScoreHand().ToString());
        }

        void IBlackJackUI.ShowPlayerBlackJack()
        {
            showMessage("Player Blackjack");
        }

        void IBlackJackUI.ShowDealerBlackJack()
        {
            showMessage("Dealer Blackjack");
        }

        void IBlackJackUI.ShowPush()
        {
            showMessage("Push: " + 
                game.player.ScoreHand().ToString());
        }

        #endregion
  
        public void ShowBets()
        {
            potLabel.Text = "Pot : " + game.Pot.ToString();
            betLabel.Text = "Bet : " + game.Bet.ToString();
        }

        public void ShowHandsPlayerTurn(Graphics g)
        {
            DrawHand(g, game.dealer, 0, 30, 80, 0);
            DrawHand(g, game.player, 0, 130, 20, 0);
            showMessage("Player : " + game.player.ScoreHand());
        }

        public void ShowHandsDealerTurn(Graphics g)
        {
            DrawHand(g, game.dealer, 0, 30, 20, 0);
            DrawHand(g, game.player, 0, 130, 20, 0);
            showMessage("Player : " + 
                game.player.ScoreHand() + 
                " Dealer : " + game.dealer.ScoreHand());
        }

        private void BlackJackWindowsForm_Load(object sender, EventArgs e)
        {

        }

        private void BlackJackWindowsForm_Paint(object sender, PaintEventArgs e)
        {
            switch (game.Mode)
            {
                case GameMode.SettingBet:
                    ShowBets();
                    break;
                case GameMode.PlayerActive:
                    ShowBets();
                    ShowHandsPlayerTurn(e.Graphics);
                    break;
                case GameMode.DealerActive:
                    ShowBets();
                    ShowHandsDealerTurn(e.Graphics);
                    break;
            }
        }

        public void DrawHand(
            Graphics g, 
            BlackJackHand hand, 
            int startx, 
            int starty, 
            int updatex,
            int updatey)
        {
            foreach (BlackJackCard card in hand.Cards)
            {
                BlackJackCardImages.DrawCardImage(g, card, startx, starty);
                startx += updatex;
                starty += updatey;
            }
        }

        private void stayButton_Click(object sender, EventArgs e)
        {
            game.DoPlayerStay();
        }

        private void hitButton_Click(object sender, EventArgs e)
        {
            game.DoHitPlayer();
        }

        private void startGameButton_Click(object sender, EventArgs e)
        {
            game.DoStartHand();
        }

        private void increaseBetButton_Click(object sender, EventArgs e)
        {
            game.DoIncreaseBet();
        }

        private void decreaseBetButton_Click(object sender, EventArgs e)
        {
            game.DoDecreaseBet();
        }

        private void newGameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            game.DoNewGame();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
    }

    public class BlackJackCardImages 
    {
        static private Image[] cardImages = new Bitmap[53];

        static private System.Drawing.Imaging.ImageAttributes transparentAttributes;
        static private System.Reflection.Assembly execAssem;
        private static Rectangle drawRect;

        static BlackJackCardImages()
        {
            transparentAttributes = new System.Drawing.Imaging.ImageAttributes();
            transparentAttributes.SetColorKey(Color.Green, Color.Green);
            execAssem = System.Reflection.Assembly.GetExecutingAssembly();
        }

        static public Image GetCardImage ( BlackJackCard card ) 
        {
            int dispNo = card.CardNo;
            if (!card.FaceUp)
            {
                dispNo = 0;
            }
            if (cardImages[dispNo] == null)
            {
                cardImages[dispNo] = new Bitmap(execAssem.GetManifestResourceStream(
                    @"BlackJackWindows.images." + dispNo + @".gif"));
            }
            return cardImages[dispNo];
        }

        static public void DrawCardImage(
            Graphics g,
            BlackJackCard card,
            int startX,
            int startY)
        {
            Image cardImage = BlackJackCardImages.GetCardImage(card);
            drawRect.X = startX;
            drawRect.Y = startY;
            drawRect.Width = cardImage.Width;
            drawRect.Height = cardImage.Height;
            g.DrawImage(
                cardImage,			    // Image
                drawRect,			    // destination rectange
                0,						// srcX
                0,						// srcY
                cardImage.Width,        // srcWidth
                cardImage.Height,	    // srcHeight
                GraphicsUnit.Pixel,		// srcUnit
                transparentAttributes); // ImageAttributes
        }
    }
}