using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

namespace BlackJackBusinessObjects
{

    public class CardShoe
    {
        private int noOfDecks = 1;

        private byte[] decks;

        private int nextCard;

        private bool testShoe = false;

        public bool TestShoe
        {
            get
            {
                return testShoe;
            }
        }

        private void makeShoe()
        {
            decks = new byte[noOfDecks * 52];
            int cardPos = 0;
            for (int i = 0; i < noOfDecks; i++)
            {
                for (byte j = 1; j < 53; j++)
                {
                    decks[cardPos] = j;
                    cardPos++;
                }
            }
            nextCard = 0;
        }

        private void shuffleShoe()
        {
            System.Random rand = new Random();
            byte swap;
            int p1, p2;
            for (int i = 0; i < decks.Length; i++)
            {
                p1 = rand.Next(decks.Length);
                p2 = rand.Next(decks.Length);
                swap = decks[p1];
                decks[p1] = decks[p2];
                decks[p2] = swap;
            }
            nextCard = 0;
        }

        public byte NextCardNo()
        {
            if (nextCard == decks.Length)
            {
                shuffleShoe();
            }
            return decks[nextCard++];
        }

        public CardShoe(int noOfDecks)
        {
            this.noOfDecks = noOfDecks;
            makeShoe();
            shuffleShoe();
            testShoe = false;
        }

        public CardShoe()
            : this(1)
        {
        }

        public CardShoe(byte[] stackedDeck)
        {
            decks = stackedDeck;
            testShoe = true;
        }
    }

    public class BlackJackHand
    {
        public ArrayList Cards = new ArrayList();

        public int ScoreHand()
        {
            int score = 0;
            int aces = 0;
            foreach (BlackJackCard card in Cards)
            {
                score += card.Score;
                if (card.Score == 11)
                {
                    aces++;
                }
            }
            while ((score > 21) && (aces > 0))
            {
                score -= 10;
                aces--;
            }
            return score;
        }

    }

    public class BlackJackCard
    {
        public byte CardNo;
        public bool FaceUp = true;

        static private byte[] scores =
            new byte[] { 	11,	//ace
							2,3,4,5,6,7,8,9,10, //spot cards
							10,10,10}; //jack, queen, king

        static private bool[] isPicture =
            new bool[] { 	false,	//ace
							false,false,false,false,false,false,false,false,false, //spot cards
							true,true,true}; //jack, queen, king
        public int Score
        {
            get
            {
                return scores[(CardNo - 1) % 13];
            }
        }

        public bool IsPicture
        {
            get
            {
                return isPicture[(CardNo - 1) % 13];
            }
        }

        public BlackJackCard(byte cardNo, bool faceUp)
        {
            CardNo = cardNo;
            FaceUp = faceUp;
        }

        public BlackJackCard(byte cardNo)
            : this(cardNo, true)
        {
        }
    }

    public interface IBlackJackUI
    {
        // methods to request a redraw
        void DoRedraw();

        // methods to inform the UI window of state changes
        void DoSettingBet();
        void ShowBets();
        void DoStartPlayerHand();
        void DoStartDealerHand();
        void ShowPlayerBust();
        void ShowDealerBust();
        void ShowPlayerWins();
        void ShowPlayerLoses();
        void ShowPlayerBlackJack();
        void ShowDealerBlackJack();
        void ShowPush();
    }

    public interface IBlackJackGame
    {
        void DoDecreaseBet();
        void DoHitPlayer();
        void DoIncreaseBet();
        void DoNewGame();
        void DoPlayerBust();
        void DoPlayerStay();
        void DoStartHand();
        GameMode Mode { get; }
        BlackJackHand player { get; }
        BlackJackHand dealer { get; }
        double Pot { get; set; }
        int Bet { get; set; }
    }


    public enum GameMode
    {
        SettingBet,
        PlayerActive,
        DealerActive
    }

    public class BlackJackGame : BlackJackBusinessObjects.IBlackJackGame 
    {

        GameMode modeValue = GameMode.SettingBet;

        private GameMode mode
        {
            get
            {
                return modeValue;
            }
            set
            {
                switch (value)
                {
                    case GameMode.SettingBet:
                        gameGUI.DoSettingBet();
                        break;
                    case GameMode.PlayerActive:
                        gameGUI.DoStartPlayerHand();
                        break;
                    case GameMode.DealerActive:
                        gameGUI.DoStartDealerHand();
                        break;
                }
                modeValue = value;
            }
        }

        public GameMode Mode
        {
            get
            {
                return modeValue;
            }
        }

        private BlackJackHand playerValue = new BlackJackHand();

        public BlackJackHand player {
            get {
                return playerValue;
            }
        }

        private BlackJackHand dealerValue = new BlackJackHand();

        public BlackJackHand dealer
        {
            get
            {
                return dealerValue;
            }
        }

        public CardShoe shoe;
        public BlackJackCard dealerHoleCard;

        #region PotManagement

        public int betChangeValue;
        private int betValue;

        public int Bet
        {
            get { return betValue; }
            set { betValue = value; }
        }


        private double potValue;

        public double Pot
        {
            get
            {
                return potValue;
            }
            set
            {
                potValue = value;
            }
        }

        public const double INITIAL_POT_VALUE = 500;
        public const int INITIAL_BET_CHANGE_VALUE = 5;

        private void resetPot()
        {
            betChangeValue = INITIAL_BET_CHANGE_VALUE;
            Bet = INITIAL_BET_CHANGE_VALUE;
            Pot = INITIAL_POT_VALUE;
        }

        public void DoIncreaseBet()
        {
            if (mode == GameMode.SettingBet)
            {
                if ((Bet + betChangeValue) < Pot)
                {
                    Bet = Bet + betChangeValue;
                    gameGUI.ShowBets();
                }
            }
        }

        public void DoDecreaseBet()
        {
            if (mode == GameMode.SettingBet)
            {
                if (Bet >= betChangeValue)
                {
                    Bet = Bet - betChangeValue;
                    gameGUI.ShowBets();
                }
            }
        }

        private void DoWinBet()
        {
            // win back 2 * our stake 
            Pot = Pot + Bet;
            Pot = Pot + Bet;
            gameGUI.DoRedraw();
        }

        private void DoLoseBet()
        {
        }

        private void DoPushBet()
        {
            // put the bet back in the pot
            Pot = Pot + Bet;
            gameGUI.DoRedraw();
        }

        private void DoPlaceBet()
        {
            Pot = Pot - Bet;
        }

        #endregion


        public void DoNewGame()
        {
            resetPot();
            mode = GameMode.SettingBet;
            gameGUI.DoRedraw();
        }

        public void DoStartHand()
        {
            // take the players bet out of the pot
            DoPlaceBet();

            // deal a new hand
            player.Cards.Clear();
            dealer.Cards.Clear();

            player.Cards.Add(new BlackJackCard(shoe.NextCardNo()));
            player.Cards.Add(new BlackJackCard(shoe.NextCardNo()));

            dealerHoleCard = new BlackJackCard(shoe.NextCardNo(), false);
            dealer.Cards.Add(dealerHoleCard);
            dealer.Cards.Add(new BlackJackCard(shoe.NextCardNo()));

            if (dealer.ScoreHand() == 21)
            {
                if (player.ScoreHand() == 21)
                {
                    gameGUI.ShowPush();
                    DoPushBet();
                }
                else
                {
                    gameGUI.ShowDealerBlackJack();
                    DoLoseBet();
                }
                mode = GameMode.SettingBet;
            }

            if (player.ScoreHand() == 21)
            {
                gameGUI.ShowPlayerBlackJack();
                DoWinBet();
                mode = GameMode.SettingBet;
            }
            mode = GameMode.PlayerActive;
            gameGUI.DoRedraw();
        }

        public void DoHitPlayer()
        {
            if (mode == GameMode.PlayerActive)
            {
                player.Cards.Add(new BlackJackCard(shoe.NextCardNo()));
                gameGUI.DoRedraw();
                if (player.ScoreHand() == 21)
                {
                    DoPlayerStay();
                }
                if (player.ScoreHand() > 21)
                {
                    DoPlayerBust();
                }
            }
        }

        public void DoPlayerBust()
        {
            if (mode == GameMode.PlayerActive)
            {
                mode = GameMode.DealerActive;
                DoLoseBet();
                gameGUI.ShowPlayerBust();
                mode = GameMode.SettingBet;
            }
        }

        public void DoPlayerStay()
        {
            if (mode == GameMode.PlayerActive)
            {
                dealerHoleCard.FaceUp = true;
                mode = GameMode.DealerActive;
                gameGUI.DoRedraw();
                System.Threading.Thread.Sleep(750);

                // draw cards for the dealer
                while (dealer.ScoreHand() < 17)
                {
                    dealer.Cards.Add(new BlackJackCard(shoe.NextCardNo()));
                    gameGUI.DoRedraw();
                    System.Threading.Thread.Sleep(750);
                }

                // decide who wins or loses
                if (player.ScoreHand() > 21)
                {
                    DoLoseBet();
                    gameGUI.ShowPlayerBust();
                    mode = GameMode.SettingBet;
                    return;
                }

                if (dealer.ScoreHand() > 21)
                {
                    DoWinBet();
                    gameGUI.ShowDealerBust();
                    mode = GameMode.SettingBet;
                    return;
                }

                if (player.ScoreHand() > dealer.ScoreHand())
                {
                    DoWinBet();
                    gameGUI.ShowPlayerWins();
                    mode = GameMode.SettingBet;
                    return;
                }

                if (player.ScoreHand() < dealer.ScoreHand())
                {
                    DoLoseBet();
                    gameGUI.ShowPlayerLoses();
                    mode = GameMode.SettingBet;
                    return;
                }

                if (player.ScoreHand() == dealer.ScoreHand())
                {
                    DoPushBet();
                    gameGUI.ShowPush();
                    mode = GameMode.SettingBet;
                    return;
                }
            }
        }

        private IBlackJackUI gameGUI;

        public BlackJackGame(IBlackJackUI inGameGUI, CardShoe inShoe)
        {
            shoe = inShoe;
            gameGUI = inGameGUI;
            DoNewGame();
        }

        public BlackJackGame(IBlackJackUI inGameGUI)
            :
            this(inGameGUI, new CardShoe(1))
        {
        }
    }
}
