using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using BlackJackBusinessObjects;

namespace BlackJackSmartphone
{
    public partial class BlackJackSmartphone : Form, IBlackJackUI
    {
        IBlackJackGame game;

        public BlackJackSmartphone()
        {
            InitializeComponent();
            game = new BlackJackGame(this);
        }

        private void showMessage(string message)
        {
            this.Text = message;
        }

        public void ShowHandsPlayerTurn(Graphics g)
        {
            DrawHand(g, game.dealer, 0, 0, 80, 0);
            DrawHand(g, game.player, 0, 110, 20, 0);
            showMessage("Player : " + game.player.ScoreHand());
        }

        public void ShowHandsDealerTurn(Graphics g)
        {
            DrawHand(g, game.dealer, 0, 0, 20, 0);
            DrawHand(g, game.player, 0, 110, 20, 0);
            showMessage("Player : " +
                game.player.ScoreHand() +
                " Dealer : " + game.dealer.ScoreHand());
        }

        #region IPocketJackUI Members

        void IBlackJackUI.DoRedraw()
        {
            Refresh();
        }

        void IBlackJackUI.DoSettingBet()
        {
            decreaseBetMenuItem.Enabled = true;
            increaseBetMenuItem.Enabled = true;
            hitMenuItem.Enabled = false;

            stayMenuItem.Text = "Start";
        }

        void IBlackJackUI.ShowBets()
        {
            showMessage("Pot : " + game.Pot.ToString() +
                " Bet : " + game.Bet.ToString());
        }

        void IBlackJackUI.DoStartPlayerHand()
        {
            decreaseBetMenuItem.Enabled = false;
            increaseBetMenuItem.Enabled = false;
            hitMenuItem.Enabled = true;

            stayMenuItem.Text = "Stay";
            showMessage("");
        }

        void IBlackJackUI.DoStartDealerHand()
        {
            decreaseBetMenuItem.Enabled = false;
            increaseBetMenuItem.Enabled = false;
            hitMenuItem.Enabled = false;

            stayMenuItem.Text = "Stay";
            showMessage("");
        }

        void IBlackJackUI.ShowPlayerBust()
        {
            showMessage("Player Bust: " + game.player.ScoreHand().ToString());
        }

        void IBlackJackUI.ShowDealerBust()
        {
            showMessage("Dealer Bust: " +
                game.dealer.ScoreHand().ToString());
        }

        void IBlackJackUI.ShowPlayerWins()
        {
            showMessage("Player wins: " +
                game.player.ScoreHand().ToString() +
                " vs " + game.dealer.ScoreHand().ToString());
        }

        void IBlackJackUI.ShowPlayerLoses()
        {
            showMessage("Player loses: " +
                game.player.ScoreHand().ToString() + " vs " +
                game.dealer.ScoreHand().ToString());
        }

        void IBlackJackUI.ShowPlayerBlackJack()
        {
            showMessage("Player Blackjack");
        }

        void IBlackJackUI.ShowDealerBlackJack()
        {
            showMessage("Dealer Blackjack");
        }

        void IBlackJackUI.ShowPush()
        {
            showMessage("Push: " +
                game.player.ScoreHand().ToString());
        }

        #endregion

        public void DrawHand(
            Graphics g,
            BlackJackHand hand,
            int startx,
            int starty,
            int updatex,
            int updatey)
        {
            foreach (BlackJackCard card in hand.Cards)
            {
                BlackJackCardImages.DrawCardImage(g, card, startx, starty);
                startx += updatex;
                starty += updatey;
            }
        }

        private void doUpKey()
        {
            switch (game.Mode)
            {
                case GameMode.DealerActive:
                    break;
                case GameMode.PlayerActive:
                    break;
                case GameMode.SettingBet:
                    game.DoIncreaseBet();
                    break;
            }
        }

        private void doDownKey()
        {
            switch (game.Mode)
            {
                case GameMode.DealerActive:
                    break;
                case GameMode.PlayerActive:
                    break;
                case GameMode.SettingBet:
                    game.DoDecreaseBet();
                    break;
            }
        }

        private void doLeftKey()
        {
            switch (game.Mode)
            {
                case GameMode.DealerActive:
                    break;
                case GameMode.PlayerActive:
                    break;
                case GameMode.SettingBet:
                    break;
            }
        }

        private void doRightKey()
        {
            switch (game.Mode)
            {
                case GameMode.DealerActive:
                    break;
                case GameMode.PlayerActive:
                    break;
                case GameMode.SettingBet:
                    break;
            }
        }

        private void doEnterKey()
        {
            switch (game.Mode)
            {
                case GameMode.DealerActive:
                    break;
                case GameMode.PlayerActive:
                    game.DoHitPlayer();
                    break;
                case GameMode.SettingBet:
                    game.DoStartHand();
                    break;
            }
        }


        private void doHitMenuItem()
        {
            switch (game.Mode)
            {
                case GameMode.DealerActive:
                    break;
                case GameMode.PlayerActive:
                    game.DoHitPlayer();
                    break;
                case GameMode.SettingBet:
                    break;
            }
        }

        private void doLeftMenuItem()
        {
            switch (game.Mode)
            {
                case GameMode.DealerActive:
                    break;
                case GameMode.PlayerActive:
                    game.DoPlayerStay();
                    break;
                case GameMode.SettingBet:
                    game.DoStartHand();
                    break;
            }
        }

        private void BlackJackSmartphone_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyCode == System.Windows.Forms.Keys.F1))
            {
                // Soft Key 1
                // Not handled when menu is present.
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F2))
            {
                // Soft Key 2
                // Not handled when menu is present.
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Up))
            {
                // Up
                doUpKey();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Down))
            {
                // Down
                doDownKey();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Left))
            {
                // Left
                doLeftKey();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Right))
            {
                // Right
                doRightKey();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Enter))
            {
                // Enter
                doEnterKey();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D1))
            {
                // 1
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D2))
            {
                // 2
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D3))
            {
                // 3
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D4))
            {
                // 4
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D5))
            {
                // 5
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D6))
            {
                // 6
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D7))
            {
                // 7
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D8))
            {
                // 8
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D9))
            {
                // 9
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F8))
            {
                // *
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D0))
            {
                // 0
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F9))
            {
                // #
            }

        }

        private void BlackJackSmartphone_Paint(object sender, PaintEventArgs e)
        {
            switch (game.Mode)
            {
                case GameMode.SettingBet:
                    break;
                case GameMode.PlayerActive:
                    ShowHandsPlayerTurn(e.Graphics);
                    break;
                case GameMode.DealerActive:
                    ShowHandsDealerTurn(e.Graphics);
                    break;
            }
        }

        private void newGameMenuItem_Click(object sender, EventArgs e)
        {
            game.DoNewGame();
        }

        private void exitMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void hitMenuItem_Click(object sender, EventArgs e)
        {
            doHitMenuItem();
        }

        private void stayMenuItem_Click(object sender, EventArgs e)
        {
            doLeftMenuItem();
        }

        private void increaseBetMenuItem_Click(object sender, EventArgs e)
        {
            game.DoDecreaseBet();
        }

        private void decreaseBetMenuItem_Click(object sender, EventArgs e)
        {
            game.DoDecreaseBet();
        }
    }

    public class BlackJackCardImages
    {
        static private Image[] cardImages = new Bitmap[53];

        static private System.Drawing.Imaging.ImageAttributes transparentAttributes;
        static private System.Reflection.Assembly execAssem;
        private static Rectangle drawRect;

        static BlackJackCardImages()
        {
            transparentAttributes = new System.Drawing.Imaging.ImageAttributes();
            transparentAttributes.SetColorKey(Color.Green, Color.Green);
            execAssem = System.Reflection.Assembly.GetExecutingAssembly();
        }

        static public Image GetCardImage(BlackJackCard card)
        {
            int dispNo = card.CardNo;
            if (!card.FaceUp)
            {
                dispNo = 0;
            }
            if (cardImages[dispNo] == null)
            {
                cardImages[dispNo] = new Bitmap(execAssem.GetManifestResourceStream(
                    @"BlackJackSmartphone.images." + dispNo + @".gif"));
            }
            return cardImages[dispNo];
        }

        static public void DrawCardImage(
            Graphics g,
            BlackJackCard card,
            int startX,
            int startY)
        {
            Image cardImage = BlackJackCardImages.GetCardImage(card);
            drawRect.X = startX;
            drawRect.Y = startY;
            drawRect.Width = cardImage.Width;
            drawRect.Height = cardImage.Height;
            g.DrawImage(
                cardImage,			    // Image
                drawRect,			    // destination rectange
                0,						// srcX
                0,						// srcY
                cardImage.Width,        // srcWidth
                cardImage.Height,	    // srcHeight
                GraphicsUnit.Pixel,		// srcUnit
                transparentAttributes); // ImageAttributes
        }
    }
}