using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Microsoft.WindowsMobile.PocketOutlook;

namespace AddressBookRoulette
{
    public partial class AddressBookRouletteForm : Form
    {
        private Sound clickSound;
        private Sound winSound;
        readonly System.Reflection.Assembly assembly =
            System.Reflection.Assembly.GetExecutingAssembly();

        public AddressBookRouletteForm()
        {
            InitializeComponent();
            clickSound = new Sound(assembly.GetManifestResourceStream("AddressBookRoulette.click.wav"));
            winSound = new Sound(assembly.GetManifestResourceStream("AddressBookRoulette.burp.wav"));

            findMobileContacts();
        }

        /// <summary>
        /// The outlook session we are going to use.
        /// </summary>
        Microsoft.WindowsMobile.PocketOutlook.OutlookSession outlook =
            new Microsoft.WindowsMobile.PocketOutlook.OutlookSession();

        /// <summary>
        /// Random number generator.
        /// </summary>
        private Random rand = new Random(1);

        /// <summary>
        /// The currently selected contact.
        /// </summary>
        private Contact contact = null;

        /// <summary>
        /// The message that we are going to send.
        /// </summary>
        private string message = "";

        /// <summary>
        /// Use generics to create a typed list of contacts. Just as useful as an
        /// ArrayList, but no need to cast the items in it.
        /// </summary>
        System.Collections.Generic.List<Contact> contactsWithMobilesList = 
            new System.Collections.Generic.List<Contact>();

        /// <summary>
        /// Populates the list with contacts which have a mobile phone number set.
        /// </summary>
        void findMobileContacts()
        {
            foreach (Contact c in outlook.Contacts.Items)
            {
                if (c.MobileTelephoneNumber != "")
                {
                    contactsWithMobilesList.Add(c);
                }
            }
        }

        /// <summary>
        /// Picks a random contact with a mobile phone from the list. 
        /// Updates the contact label on the screen and plays sound effects.
        /// </summary>
        /// <returns>selected contact</returns>
        private Contact pickContact()
        {
            int contactNo = 0;

            Contact result = null;

            int step = 10;

            for (int delay = 20; delay < 1000; delay += step)
            {
                clickSound.Play();
                System.Threading.Thread.Sleep(delay);
                step += (step / 4);

                // find a random contact with a mobile phone
                contactNo = rand.Next(contactsWithMobilesList.Count);
                result = contactsWithMobilesList[contactNo];

                contactLabel.Text = result.ToString();
                contactLabel.Refresh();
            }
            winSound.Play();

            return result;
        }

        /// <summary>
        /// Messages. Base message is the one which is selected if you cheat
        /// </summary>
        private string[] messages = new string[] {
            "You are the greatest",
            "Get Lost",
            "I love You",
            "Marry Me",
            "Fancy a Drink?",
            "I know your secret"
        };

        /// <summary>
        /// Picks a random message from the messages aray. 
        /// Updates the messageLabel and plays sound effects
        /// </summary>
        /// <returns></returns>
        private string pickMessage()
        {
            int messageNo = 0;
            int step = 10;
            for (int delay = 20; delay < 1000; delay += step)
            {
                clickSound.Play();
                System.Threading.Thread.Sleep(delay);
                step += (step / 4);
                messageNo = rand.Next(messages.Length);
                messageLabel.Text = messages[messageNo];
                messageLabel.Refresh();
            }
            winSound.Play();

            return messages[messageNo];
        }


        /// <summary>
        /// Sends the message. Displays an error if the send fails and
        /// a confirmation message if it was delivered OK.
        /// </summary>
        private void sendMessage()
        {
            SmsMessage sms =
                new SmsMessage(contact.MobileTelephoneNumber, message);
            try
            {
                sms.Send();
            }
            catch
            {
                MessageBox.Show("Send Message Failed");
                return;
            }
            MessageBox.Show("Message Sent");
        }

        /// <summary>
        /// Applies a cheat to the selection. Finds the given contact and
        /// sets this to receive the first message in the array.
        /// </summary>
        private void cheatMessage()
        {
            Cursor.Current = Cursors.WaitCursor;

            message = messages[0];
            messageLabel.Text = message;

            foreach ( Contact c in outlook.Contacts.Items ) {
                if ( c.FirstName == "Rob" && c.LastName == "Miles" ) {
                    contact = c;
                    contactLabel.Text = c.ToString();
                    break;
                }
            }

            Cursor.Current = Cursors.Default;
        }

        private void AddressBookRouletteForm_KeyDown(object sender, KeyEventArgs e)
        {
            if ((e.KeyCode == System.Windows.Forms.Keys.F1))
            {
                // Soft Key 1
                // Not handled when menu is present.
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F2))
            {
                // Soft Key 2
                // Not handled when menu is present.
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Up))
            {
                // Up
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Down))
            {
                // Down
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Left))
            {
                // Left
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Right))
            {
                // Right
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.Enter))
            {
                // Enter
                contact = pickContact();
                message = pickMessage();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D1))
            {
                // 1
                cheatMessage();
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D2))
            {
                // 2
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D3))
            {
                // 3
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D4))
            {
                // 4
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D5))
            {
                // 5
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D6))
            {
                // 6
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D7))
            {
                // 7
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D8))
            {
                // 8
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D9))
            {
                // 9
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F8))
            {
                // *
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.D0))
            {
                // 0
            }
            if ((e.KeyCode == System.Windows.Forms.Keys.F9))
            {
                // #
            }

        }

        private void sendMenuItem_Click(object sender, EventArgs e)
        {
            sendMessage();
        }

        private void doneMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
    }
}